/* Basic fnvrand periodicity-testing program.  Tests for recurrance of
   the first N bytes of the number stream in a later part of the
   stream.  If N is large enough then it is statistically likely
   that such a recurrance indicates a periodic loop.
*/
/* n.b. This program uses 'long long', which isn't standard ANSI C. */

#include <stdio.h>
#include <string.h>

#include "fnvrand.h"

/* size of recurrance buffer measured in 32-bit words */
#define NSIZE 5
static unsigned long nwords[NSIZE];
static unsigned long wwords[NSIZE+1];
static char* wbytes = (char*)wwords;

int main(int argc, char *argv[])
{
  unsigned long long bytes_done = 0;
  int i;

  fnvsrand(0); /* In theory, the seed does affect the periodicity. */

  printf("Testing for byte-level periodity in fnvrand32() stream.\n");
  printf("NOTE: Probability of a false-positive periodicy report is approx\n"
         "1 in 2^%d for every byte considered.\n", 32 * NSIZE);

  for (i=0; i<NSIZE; ++i) {
    wwords[i] = nwords[i] = fnvrand32();
    bytes_done += 4;
  }

  while (1) {
    wwords[NSIZE] = fnvrand32();
    for (i=1; i<=4; ++i) {
      if (memcmp(nwords, &wbytes[i], 4 * NSIZE) == 0) {
        printf("At byte position %lld found a run of %d bytes that match the\n"
               "start of the stream!  This is perhaps the periodicity of \n"
               "the stream (%lld words), but there is a 1 in 2^%d chance\n"
               "for every byte position considered that this is a false\n"
               "positive and that that real periodicy is still greater.\n",
               bytes_done, 4 * NSIZE, bytes_done/4, 32 * NSIZE);
        return 0;
      }
      ++bytes_done;
      if ((bytes_done & (0x07FFFFFF)) == 0) {
        printf("Passed %lld 32-bit words without repetition...\n", bytes_done/4);
      }
    }
    memmove(wwords, &wwords[1], 4 * NSIZE);
  }
}
